SHADER version 1
@OpenGL2.Vertex
#version 400
#define MAX_INSTANCES 256

//Uniforms
uniform instancematrices { mat4 matrix[MAX_INSTANCES];} entity;
uniform mat4 projectioncameramatrix;

//Attributes
in vec3 vertex_position;
in vec2 vertex_texcoords0;
in vec3 vertex_normal;
in vec3 vertex_binormal;
in vec3 vertex_tangent;

//Outputs
out vec2 ex_texcoords0;
out vec3 ex_normal;
out vec3 ex_tangent;
out vec3 ex_binormal;
out vec4 ex_vertexposition;
out vec4 glpos;

void main()
{
	mat4 entitymatrix = entity.matrix[gl_InstanceID];
	mat4 entitymatrix_=entitymatrix;
	entitymatrix_[0][3]=0.0;
	entitymatrix_[1][3]=0.0;
	entitymatrix_[2][3]=0.0;
	entitymatrix_[3][3]=1.0;
	
	ex_vertexposition = entitymatrix_ * vec4(vertex_position,1.0);
	gl_Position = projectioncameramatrix * ex_vertexposition;
	
	//Pass ex_vertexposition as gl_Position
	glpos = gl_Position;

	mat3 nmat = mat3(entitymatrix_);
	ex_normal = normalize(nmat * vertex_normal);	
	ex_tangent = normalize(nmat * vertex_tangent);
	ex_binormal = normalize(nmat * vertex_binormal);
	
	ex_texcoords0 = vertex_texcoords0;
}
@OpenGLES2.Vertex

@OpenGLES2.Fragment

@OpenGL4.Vertex
#version 400
#define MAX_INSTANCES 256

//Uniforms
uniform instancematrices { mat4 matrix[MAX_INSTANCES];} entity;
uniform mat4 projectioncameramatrix;

//Attributes
in vec3 vertex_position;
in vec2 vertex_texcoords0;
in vec3 vertex_normal;
in vec3 vertex_binormal;
in vec3 vertex_tangent;

//Outputs
out vec2 ex_texcoords0;
out vec3 ex_normal;
out vec3 ex_tangent;
out vec3 ex_binormal;
out vec4 ex_vertexposition;
out vec4 glpos;

void main()
{
	mat4 entitymatrix = entity.matrix[gl_InstanceID];
	mat4 entitymatrix_=entitymatrix;
	entitymatrix_[0][3]=0.0;
	entitymatrix_[1][3]=0.0;
	entitymatrix_[2][3]=0.0;
	entitymatrix_[3][3]=1.0;
	
	ex_vertexposition = entitymatrix_ * vec4(vertex_position,1.0);
	gl_Position = projectioncameramatrix * ex_vertexposition;
	
	//Pass ex_vertexposition as gl_Position
	glpos = gl_Position;

	mat3 nmat = mat3(entitymatrix_);
	ex_normal = normalize(nmat * vertex_normal);	
	ex_tangent = normalize(nmat * vertex_tangent);
	ex_binormal = normalize(nmat * vertex_binormal);
	
	ex_texcoords0 = vertex_texcoords0;
}
@OpenGL4.Fragment
#version 400
#define BFN_ENABLED 1

//Variables
float ripple_speed = 5.5;

//Uniforms
uniform sampler2D texture0;//diffuse map
uniform sampler2D texture1;//normal map
//uniform samplerCube texture7;//skybox
uniform sampler2D texture10;//refraction map
uniform sampler2DMS texture11;//depth map
uniform vec2 buffersize;
uniform bool isbackbuffer;
uniform vec2 camerarange;
uniform float currenttime;
float time = currenttime/1000;
uniform mat4 projectionmatrix;
uniform vec3 cameraposition;

//Inputs
in vec2 ex_texcoords0;
in vec4 ex_vertexposition;
in vec3 ex_normal;
in vec3 ex_tangent;
in vec3 ex_binormal;
in vec4 glpos;

out vec4 fragData0;
out vec4 fragData1;

float depthToPosition(in float depth, in vec2 depthrange)
{
	return depthrange.x / (depthrange.y - depth * (depthrange.y - depthrange.x)) * depthrange.y;
}

void main(void)
{

	//Texture scroll
	vec2 tex = ex_texcoords0;
	tex.y -= time * 0.2; // down 
	
	//Diffuse
	vec4 outcolor = texture(texture0,ex_texcoords0);

	//Normals
	vec3 normal = ex_normal;
	normal =  normalize(texture(texture1,1.0 * tex * 0.8 + (time/ripple_speed)).xyz * 2.0 - 1.0);
	normal += normalize(texture(texture1,1.2 * tex * 0.8 - (time/ripple_speed)).xyz * 2.0 - 1.0);
	normal.z /= 0.5;
	normal = ex_tangent*normal.x + ex_binormal*normal.y + ex_normal*normal.z;
	normal=normalize(normal);

	//Refraction
	//-----------------------------------------------------------------------------
	vec2 screencoord = vec2((gl_FragCoord.xy/buffersize*0.9) + normal.xy * 0.05);
	screencoord.x += 0.0055;
	screencoord.y -= 0.05;
	screencoord *= 1.1;
	if (isbackbuffer) screencoord.y = 1.0 - screencoord.y;
	
	//Check depth buffer
	if (glpos.z > depthToPosition(texelFetch(texture11,ivec2(screencoord*buffersize),0).x,camerarange))
	{
		screencoord = vec2((gl_FragCoord.xy/buffersize));
		if (isbackbuffer) screencoord.y = 1.0-screencoord.y;		
	}
	
	vec4 refraction = texture(texture10, screencoord);
	//-----------------------------------------------------------------------------

	//Output Diffuse
	fragData0 = mix(outcolor,refraction,outcolor.a);
	
	fragData1 = vec4(normalize(normal)*0.5+0.5,fragData0.a);
	fragData1.a = 1/255.0;
}
